<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\ThirdPlatform\Api;

use App\ImportExport\WuxiLab\WuxiLabExport;
use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;


class WuxiApiNewService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('wuxi_appTec');
        $this->getPlatformInfo();
    }

    /**
     * 批量更新商品信息
     * @param $product
     * @return bool
     */
    public function pushBatchUpdateProduct($product)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['chemicalProducts'] = $this->mergeApiMetaData($product);

                $response = $this->getPostClient($this->platformInfo['platform_url'].'/api/v2/updateChamical',$params, null,
                    [
                            'authorization' => $this->getAuthHeader()
                    ]
                );

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    /**
     * @更新包装规格
     * @param $packages
     * @param $token
     * @return false|mixed|void
     */
    public function pushBatchUpdatePackages($packages,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['company_id'] = $this->platformInfo['platform_params']['companyId'];
                $params['product_type'] = 'CHAMICAL';
                $params = array_merge($params,$packages);

                $response = $this->getPostClient($this->platformInfo['platform_params']['adminUrl'].'/package/specification/save',$params,null, [
                    //'cookie' => $this->getCookie($token)
                ]);

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    public function getPackageList($token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['companyId'] = $this->platformInfo['platform_params']['companyId'];
                $params['productType'] = 'CHAMICAL';

                $urlParams = [];
                foreach($params as $key => $param){
                    $urlParams[] = $key.'='.$param;
                }

                $response = $this->clientRequest('get',
                    $this->platformInfo['platform_params']['adminUrl'].'/package/specification/count?'.implode('&',$urlParams),
                    [
                        'headers' => [
                            //'cookie' => $this->getCookie($token),
                            'content-type' => 'application/json;charset=UTF-8'
                        ]
                    ]);
                if($response && $response > 0){
                    $page = 1;
                    $count = ceil($response / 30);

                    $packageList = [];

                    while(true){
                        $params['page'] = $page;
                        $params['rows'] = 30;
                        $urlParams = [];
                        foreach($params as $key => $param){
                            $urlParams[] = $key.'='.$param;
                        }
                        $packageListResponse = $this->clientRequest('get',
                            $this->platformInfo['platform_params']['adminUrl'].'/package/specification/list?'.implode('&',$urlParams),
                            [
                                'headers' => [
                                    //'cookie' => $this->getCookie($token),
                                    'content-type' => 'application/json;charset=UTF-8'
                                ]
                            ]);

                        $result = $this->apiResponse($packageListResponse);
                        if($result){
                            $packageList = array_merge($packageList,$result);
                        }

                        $page ++;
                        if($page > $count){
                            break;
                        }
                        sleep(2);
                    }

                    return $packageList;
                }

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    /**
     * 整理组装 api 数据
     * @param $metaData
     * @return array
     */
    public function mergeApiMetaData($metaData)
    {
        if(empty($metaData)){
            return [];
        }

        $mergeData = [];
        foreach($metaData['rawList'] as $datas){
            $tempData['supplier_product_id'] = $datas['r_code'];
            $tempData['mdl'] = '';
            $tempData['cas'] = $datas['c_cas'];
            $tempData['nu'] = '';
            $tempData['chinese_name'] = $datas['r_cn_name'];
            $tempData['english_name'] = $datas['r_en_name'];
            $tempData['chemicalformula'] = $datas['c_fzs'] ?? '';
            $tempData['purity'] = $datas['r_level'];
            /*$tempData['usually_ships_days_min'] = '';
            $tempData['usually_ships_within_days'] = '';
            $tempData['backorder_lead_time'] = '';*/
            //储存条件
            switch($datas['c_cctj']){
                case 1:  $tempData['storage_condition'] = 0;break;
                case 2:  $tempData['storage_condition'] = 1;break;
                case 3:  $tempData['storage_condition'] = 2;break;
            }

            //运输条件
            switch ($tempData['storage_condition']){
                case 0: $tempData['transportation_condition'] = 0;break;
                case 1: $tempData['transportation_condition'] = 2;break;
                case 2: $tempData['transportation_condition'] = 1;break;
            }

            /*产品种类
            1: Templates/Scaffolds，
2: Amino Acids &
Peptides，
3：Building Blocks，
4：Screening Library
Compounds，
5：Fragment Library
Compounds，
6：Reagent Services，
7：Natural Products，
8：Impurity
            */
            $tempData['group_id'] = 6;
            //是否进口
            $tempData['is_imported'] = false;
            //是否推送电商
            $tempData['is_push_ds'] = false;

            //$tempData['status'] = $datas['r_status'] == '1' ? 'ACTIVE' : 'INACTIVE';

            $tempData['inventorys'][] = [
                'type' => $datas['r_unit'],
                'quantity' => floatval($datas['r_stock']),
                'country' => 'CN',
                'stock_status' => $datas['r_stock'] > 0 ? 'INSTOCK' : 'OUTSTOCK',
            ];

            $tempData['packages'] = [];

            if(isset($metaData['packageList'][$datas['r_code']])){
                $package = $metaData['packageList'][$datas['r_code']];

                $pack['unit_description'] = $package['p_pack_unit'];
                //$pack['package_status'] = $package['p_status'] == '1' && $package['p_show'] == '1' && $package['p_if_big'] == 0 ? 'ACTIVE' : 'INACTIVE';
                //$tempData['status'] = $package['p_status'] == '1' && $package['p_show'] == '1' && $package['p_if_big'] == 0 ? 'ACTIVE' : 'INACTIVE';

                $pack['inventorys'] = [
                    'package_quantity' => $package['p_stock'],
                    //'stock_status' => $package['p_stock'] > 0 ? 'INSTOCK' : 'OUTSTOK',
                    'country' => 'CN'
                ];

                $pack['prices'] = [
                    'price' => $package['p_price'],
                    'currency' => 'RMB',
                    //'status' => 'ACTIVE'
                ];

                /*$pack['costs'] = [
                    'In_cost' => $package['p_avg_cost'],
                    'In_cost_currency' => 'RMB',
                    'status' => $package['p_stock'] > 0 ? 'ACTIVE' : 'INACTVIE',
                ];*/

                $pack['wuxiPackagePricings'] = [
                    'discount'=> 0.00,
                    'currency' => 'RMB',
                    //'status' => 'INACTIVE'
                ];

                $tempData['packages'][] = $pack;
            }

            $mergeData[] = $tempData;
            unset($tempData);
        }

        return $mergeData;
    }

    public function getAuthHeader()
    {
        $auth = $this->dataEntriesService->getEntriesValues('wuxi_api_auth');
        if(!$auth){
            $response = $this->getPostClient($this->platformInfo['platform_url'].'/token/fetch', [
                    'username' => '790087059@qq.com',
                    'password' => 'YXH1992xb!!'
                ]
            );
            $result = $this->apiResponse($response);
            if($result){
                if($result['code'] == '200' && $result['success'] == 'true'){
                    $this->dataEntriesService->storeEntries(6, 'wuxi_api_auth',
                        ['authorization' => $result['data']]
                    );
                }
            }
            return $result['data'];
        }
        return $auth['authorization'];
    }

    protected function apiResponse($response)
    {
        $decodeResponse = json_decode($response,true);
        if($decodeResponse){
            return $decodeResponse;
        }
        return false;
    }

    protected function requestError($response)
    {
        SimpleLogs::writeLog($response->getMessage(), 'WuxiApi error', 'error');
        throw $response;
    }
}
